{

Ruslan Strelba. 2001. rust@merlin.com.ua

Useful for table's structure

create table ITEM
(
ITEM_ID        integer not null primary key,
ITEM_PARENT_ID integer not null references ITEM,
ITEM_NAME      varchar(...)
)

Root values (ITEM_ID=0, ITEM_PARENT_ID=0, ITEM_NAME='ROOT'

Database      - IBX database
Transaction   - IDX transaction
RootId        - identifier of root (forexample 0)
TableName     - name of table (ITEM)    
FieldId       - name of primary key's field (ITEM_ID)
FieldParentId - name of field which references parent record (ITEM_PARENT_ID)
FieldName     - name of field which will shows in TreeView (ITEM_NAME)
FieldState    - state field (same as FieldId if no state field) (ITEM_ID)
RootName      - name of root in TreeView (Root of tree)


call tree.BuildTree for inialize tree


}




unit DBTrees;

interface

uses ComCtrls, DB, Classes, SysUtils, IBDatabase, IBCustomDataSet;

type

TTreeNodeInfo=record
   Id:Integer;
   ParentId:Integer;
end;

TOnBuildNodeProc = procedure(Sender:TObject;
                              StateFieldValue : Variant;
                              var StateImage : Integer) of object;

TDB_TreeView=class(TTreeView)
private
     //
     FId,FParentId,FName, FState :string;
     FRootId : Integer;
     FTableName : String;
     IB_Connection  : TIBDatabase;
     IB_Transaction : TIBTransaction;
     FQuery : TIBDataSet;
     FRootName : String;
     //
     FOnBuildNode : TOnBuildNodeProc;
     //
     procedure BuildNode(ParentId:Integer;ParentNode:TTreeNode);
     procedure OnExpand(Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
public
     procedure BuildTree;
     procedure ClearTree;
     //
     constructor Create(AOwner:TComponent); override;
     destructor  Destroy; virtual;
     //
     procedure RebuildNode(ParentNode:TTreeNode);
     procedure RebuildSelected;
     function GetId : Integer;
published
     property Database : TIBDatabase read IB_Connection write IB_Connection;
     property Transaction : TIBTransaction read IB_Transaction write IB_Transaction;
     property RootId : Integer read FRootId write FRootId;
     property TableName : string read FTableName write FTableName;
     property FieldId:string Read FId Write FId;
     property FieldParentId:string Read FParentId Write FParentId;
     property FieldName:string Read FName Write FName;
     property FieldState : String Read FState Write FState;
     property RootName : String read FRootName write FRootName;

     property OnBuildNode : TOnBuildNodeProc read FOnBuildNode write FOnBuildNode;
end;

procedure Register;

implementation

function TDB_TreeView.GetId : Integer;
begin
     if Assigned(Selected)
     then Result := Integer(Selected.Data)
     else Result := 0;
end;

constructor TDB_TreeView.Create(AOwner:TComponent);
begin
     inherited Create(AOwner);
     //
     FQuery:=TIBDataSet.Create(Self);
     FQuery.Close;
     FQuery.Database := IB_Connection;
     FQuery.Transaction := IB_Transaction;
     //
     Self.OnExpanding:=OnExpand;
end;

procedure TDB_TreeView.OnExpand(Sender: TObject; Node: TTreeNode; var AllowExpansion: Boolean);
begin
     //Items.BeginUpdate;
     BuildNode(Integer(Node.Data),Node);
     //Items.EndUpdate;
end;

destructor TDB_TreeView.Destroy;
begin
     inherited;
     FQuery.Free;
end;

procedure TDB_TreeView.BuildNode(ParentId:Integer;ParentNode:TTreeNode);
var NodeInfo:^TTreeNodeInfo;
    parent_node:TTreeNode;
    node:TTreeNode;
    _FRootName : String;
    image_index : Integer;
begin
     if (ParentNode=nil)  then
     begin
          ClearTree;
          //
          if FRootName='' then
          begin
            FQuery.Close;
            FQuery.Database := IB_Connection;
            FQuery.Transaction := IB_Transaction;
            //
            FQuery.SelectSQL.Clear;
            FQuery.SelectSQL.Add(Format('Select %s as ROOT_NAME from %s where %s=%d',[FName,FTableName,FId,FRootId]));
            FQuery.Open;
            //
            _FRootName := FQuery.FieldByName('ROOT_NAME').AsString;
          end else _FRootName := FRootName;
          //
          parent_node:=Items.AddObject(Selected,_FRootName,NodeInfo);
          parent_node.Data:=Pointer(FRootId);
     end
     else
     begin
          ParentNode.DeleteChildren;
          parent_node:=ParentNode;
     end;
     //
     FQuery.Close;
     FQuery.Database := IB_Connection;
     FQuery.Transaction := IB_Transaction;
     //
     FQuery.SelectSQL.Clear;
     if ParentId>=0
     then FQuery.SelectSQL.Add(Format('Select c1.%s,c1.%s,c1.%s,(select count(*) from %s c2 where c2.%s=c1.%s) ChildCount From %s c1 Where c1.%s=%d and c1.%s<>%d Order by c1.%s',[FState, FId,FName,FTableName,FParentId,FId,FTableName,FParentId,ParentId,FId,FRootId,FName]))
     else FQuery.SelectSQL.Add(Format('Select c1.%s,c1.%s,c1.%s,(select count(*) from %s c2 where c2.%s=c1.%s) ChildCount From %s c1 Where c1.%s is null Order by c1.%s',[FState, FId,FName,FTableName,FParentId,FId,FTableName,FParentId,FId,FName]));
     FQuery.Open;
     //
     FQuery.First;
     while not FQuery.EOF do
     begin
          //
          node:=Items.AddChild(parent_node,FQuery.FieldByName(FName).AsString);
          node.Data:=Pointer(FQuery.FieldByName(FId).AsInteger);
          if Assigned(FOnBuildNode) then
          begin
            FOnBuildNode(Self, FQuery.Fields[0].AsVariant,image_index);
            node.ImageIndex := image_index;
          end else node.ImageIndex := -1;
          if FQuery.FieldByName('ChildCount').AsInteger>0 then Items.AddChild(node,'');
          //
          FQuery.Next;
     end;
end;

procedure TDB_TreeView.RebuildSelected;
begin
   //
   Items.BeginUpdate;
   BuildNode(Integer(Selected.Data),Selected);
   Selected.Expand(False);
   Items.EndUpdate;   
end;

procedure TDB_TreeView.BuildTree;
begin
     Items.BeginUpdate;
     BuildNode(FRootId,nil);
     Items.EndUpdate;
end;

procedure TDB_TreeView.ClearTree;
begin
     Items.BeginUpdate;
     While Items.Count>0 do Items.Delete(Items[0]);
     Items.EndUpdate;
end;

procedure TDB_TreeView.RebuildNode(ParentNode:TTreeNode);
var ParentId:Integer;
begin
     //
     BuildNode(Integer(ParentNode),ParentNode);
     //ParentNode.Expand(True);
end;


procedure Register;
begin
     RegisterComponents('Data Controls',[TDB_TreeView]);
end;

end.
